@@ -27,7 +27,6 @@
 
 
 #include "conf.h"
-#include "version.h"
 #include "mygetopt.h"
 #include "file.h"
 
@@ -472,12 +471,6 @@ static int do_option(int optc, const char *arg)
         if (!set_method(M_NRV2D_LE32, -1))
             e_method(M_NRV2D_LE32, opt->level);
         break;
-#if 0
-    case 705:
-        if (!set_method(M_NRV2E_LE32, -1))
-            e_method(M_NRV2E_LE32, opt->level);
-        break;
-#endif
 
     // compression level
     case '1':
@@ -492,7 +485,7 @@ static int do_option(int optc, const char *arg)
         if (!set_method(-1, optc - '0'))
             e_method(opt->method, optc);
         break;
-    case 900:
+    case 900:                               // --best
         if (!set_method(-1, 10))
             e_method(opt->method, 10);
         break;
@@ -514,16 +507,16 @@ static int do_option(int optc, const char *arg)
         opt->no_env = true;
         break;
     // compression settings
-    case 520:
+    case 520:                               // --small
         if (opt->small < 0)
             opt->small = 0;
         opt->small++;
         break;
-    case 521:
+    case 521:                               // --filter=
         getoptvar(&opt->filter, 0, 255);
         opt->all_filters = false;
         break;
-    case 523:
+    case 523:                               // --all-filters
         opt->all_filters = true;
         opt->filter = -1;
         break;
@@ -547,7 +540,7 @@ static int do_option(int optc, const char *arg)
         getoptvar(&opt->crp.max_match, 16u, ~0u);
         break;
     case 537:
-        if (getoptvar(&opt->crp.m_size, 10000u, 999999u) != 0)
+        if (getoptvar(&opt->crp.m_size, 10000u, (unsigned)999999u) != 0)
             e_optval("--crp-ms=");
         break;
     // backup
@@ -660,9 +653,9 @@ static int get_options(int argc, char **argv)
 static const struct mfx_option longopts[] =
 {
     // commands
-    {"best",                0, 0, 900},     // compress best
+    {"best",             0x10, 0, 900},     // compress best
     {"decompress",          0, 0, 'd'},     // decompress
-    {"fast",                0, 0, '1'},     // compress faster
+    {"fast",             0x10, 0, '1'},     // compress faster
     {"fileinfo",         0x10, 0, 902},     // display info about file
     {"help",                0, 0, 'h'+256}, // give help
     {"license",             0, 0, 'L'},     // display software license
@@ -712,17 +705,8 @@ static const struct mfx_option longopts[] =
     {"color",               0, 0, 514},
 
     // compression method
-    {"2b",               0x10, 0, 702},     // --2b
-    {"n2b",              0x10, 0, 702},     // --n2b
     {"nrv2b",            0x10, 0, 702},     // --nrv2b
-    {"2d",               0x10, 0, 704},     // --2d
-    {"n2d",              0x10, 0, 704},     // --n2d
     {"nrv2d",            0x10, 0, 704},     // --nrv2d
-#if 0
-    {"2e",               0x10, 0, 705},     // --2e
-    {"n2e",              0x10, 0, 705},     // --n2e
-    {"nrv2e",            0x10, 0, 705},     // --nrv2e
-#endif
     // compression settings
 //    {"all-filters",      0x10, 0, 523},
     {"filter",           0x31, 0, 521},     // --filter=
@@ -783,8 +767,8 @@ static void get_envoptions(int argc, char **argv)
 static const struct mfx_option longopts[] =
 {
     // commands
-    {"best",                0, 0, 900},     // compress best
-    {"fast",                0, 0, '1'},     // compress faster
+    {"best",             0x10, 0, 900},     // compress best
+    {"fast",             0x10, 0, '1'},     // compress faster
 
     // options
     {"info",                0, 0, 'i'},     // info mode
@@ -944,6 +928,10 @@ void upx_sanity_check(void)
     COMPILE_TIME_ASSERT(sizeof(void *) >= 4);
     COMPILE_TIME_ASSERT(sizeof(long) >= sizeof(void *));
 
+    COMPILE_TIME_ASSERT(sizeof(upx_int64l) >= 8);
+    COMPILE_TIME_ASSERT(sizeof(upx_int64l) >= sizeof(long));
+    COMPILE_TIME_ASSERT(sizeof(upx_int64l) == sizeof(upx_uint64l));
+
     COMPILE_TIME_ASSERT(sizeof(off_t) >= sizeof(long));
     COMPILE_TIME_ASSERT(((off_t) -1) < 0);
     COMPILE_TIME_ASSERT(sizeof(ptrdiff_t) >= sizeof(int));
@@ -963,6 +951,7 @@ void upx_sanity_check(void)
     COMPILE_TIME_ASSERT(__alignof__(LE32) == 1);
 #endif
 
+#if !defined(__WATCOMC__)
     struct align_assertion_1a_t
     {
         struct foo_t {
@@ -1004,10 +993,29 @@ void upx_sanity_check(void)
     COMPILE_TIME_ASSERT(sizeof(align_assertion_2a_t) == sizeof(align_assertion_2b_t));
     COMPILE_TIME_ASSERT(sizeof(align_assertion_1a_t) == 3*9);
     COMPILE_TIME_ASSERT(sizeof(align_assertion_2a_t) == 3*17);
+#endif
 
     COMPILE_TIME_ASSERT(sizeof(UPX_VERSION_STRING4) == 4 + 1);
     assert(strlen(UPX_VERSION_STRING4) == 4);
     assert(memcmp(UPX_VERSION_STRING4, UPX_VERSION_STRING, 4) == 0);
+
+    const unsigned char dd[4] = { 0xff, 0xfe, 0xfd, 0xfc };
+    assert(get_be16(dd) == 0xfffe);
+    assert(get_be16_signed(dd) == -2);
+    assert(get_be24(dd) == 0xfffefd);
+    assert(get_be24_signed(dd) == -259);
+    assert(get_be32(dd) == 0xfffefdfc);
+    assert(get_be32_signed(dd) == -66052);
+    assert(get_le16(dd) == 0xfeff);
+    assert(get_le16_signed(dd) == -257);
+    assert(get_le24(dd) == 0xfdfeff);
+    assert(get_le24_signed(dd) == -131329);
+    assert(get_le32(dd) == 0xfcfdfeff);
+    assert(get_le32_signed(dd) == -50462977);
+    assert(find_be16(dd, sizeof(dd), 0xfffe) == 0);
+    assert(find_le16(dd, sizeof(dd), 0xfeff) == 0);
+    assert(find_be32(dd, sizeof(dd), 0xfffefdfc) == 0);
+    assert(find_le32(dd, sizeof(dd), 0xfcfdfeff) == 0);
 }
 
 
@@ -1021,7 +1029,7 @@ int main(int argc, char *argv[])
 {
     int i;
     static char default_argv0[] = "upx";
-    int cmdline_cmd = CMD_NONE;
+//    int cmdline_cmd = CMD_NONE;
 
 #if 0 && defined(__DJGPP__)
     // LFN=n may cause problems with 2.03's _rename and mkdir under WinME
@@ -1040,10 +1048,6 @@ int main(int argc, char *argv[])
     upx_sanity_check();
     init_options(opt);
 
-#if defined(WITH_MSS)
-    MSS_DISABLE_LOG_OUTPUT;
-#endif
-
     if (!argv[0] || !argv[0][0])
         argv[0] = default_argv0;
     argv0 = argv[0];
@@ -1083,7 +1087,7 @@ int main(int argc, char *argv[])
     }
 #endif
 #if defined(WITH_NRV)
-    if (nrv_init() != NRV_E_OK)
+    if (nrv_init() != NRV_E_OK || NRV_VERSION != nrv_version())
     {
         show_head();
         fprintf(stderr,"nrv_init() failed - check your NRV installation !\n");
@@ -1107,7 +1111,7 @@ int main(int argc, char *argv[])
     assert(i <= argc);
 
     set_term(stdout);
-    cmdline_cmd = opt->cmd;
+//    cmdline_cmd = opt->cmd;
     switch (opt->cmd)
     {
     case CMD_NONE:
