@@ -234,14 +234,7 @@ unsigned optimize_relocs(upx_byte *b, const unsigned size,
     set_le16 (crel_save,ones);
     set_le16 (crel_save+2,seg_high);
 
-#if 0 // def TESTING
-    //if (opt->debug >= 3)
-    {
-        FILE *f1=fopen ("x.rel","wb");
-        fwrite (crel_save,crel-crel_save,1,f1);
-        fclose (f1);
-    }
-#endif
+    //OutputFile::dump("x.rel", crel_save, crel - crel_save);
     return crel - crel_save;
 }
 
