@@ -27,7 +27,6 @@
 
 
 #include "conf.h"
-#include "version.h"
 #include "file.h"
 #include "packer.h"
 #include "filter.h"
@@ -182,10 +181,6 @@ bool Packer::compress(upx_bytep in, upx_bytep out,
         conf.max_match = opt->crp.max_match;
     if (opt->crp.m_size != UPX_UINT_MAX && opt->crp.m_size > 10000)
         conf.m_size = opt->crp.m_size;
-#if 0 && defined(WITH_NRV)
-    else if (level >= 9)
-        conf.m_size = 32768;
-#endif
 
     // Avoid too many progress bar updates. 64 is s->bar_len in ui.cpp.
     unsigned step = (ph.u_len < 64*1024) ? 0 : ph.u_len / 64;
@@ -834,11 +829,11 @@ void Packer::initLoader(const void *pdata, int plen, int pinfo, int small)
     if (small < 0)
         small = opt->small;
     if (small >= 2)
-        addSection("IDENTSTR",identtiny,sizeof(identtiny));
+        addSection("IDENTSTR", identtiny, sizeof(identtiny));
     else if (small >= 1)
-        addSection("IDENTSTR",identsmall,sizeof(identsmall));
+        addSection("IDENTSTR", identsmall, sizeof(identsmall));
     else
-        addSection("IDENTSTR",identbig,sizeof(identbig));
+        addSection("IDENTSTR", identbig, sizeof(identbig));
 }
 
 
@@ -856,12 +851,12 @@ void Packer::addLoader(const char *s, ...)
 
 void Packer::addSection(const char *sname, const char *sdata, unsigned len)
 {
-    linker->addSection(sname,sdata,len);
+    linker->addSection(sname, sdata, len);
 }
 
 int Packer::getLoaderSection(const char *name, int *slen)
 {
-    return linker->getSection(name,slen);
+    return linker->getSection(name, slen);
 }
 
 
