@@ -28,8 +28,8 @@
 
 #include "conf.h"
 #include "file.h"
-#include "screen.h"
 #include "ui.h"
+#include "screen.h"
 #include "packer.h"
 
 
@@ -47,7 +47,7 @@ enum {
 };
 
 
-struct UiPacker::State
+struct UiPacker__State
 {
     int mode;
 
@@ -116,7 +116,7 @@ static void init_global_constants(void)
     done = true;
 
 #if 1 && defined(__DJGPP__)
-    /* check for Windows NT/2000 */
+    /* check for Windows NT/2000/XP */
     if (_get_dos_version(1) == 0x0532)
         return;
 #endif
@@ -143,12 +143,22 @@ static const char *mkline(unsigned long fu_len, unsigned long fc_len,
                           bool decompress=false)
 {
     static char buf[2000];
-    char r[7+1];
+    char r[7+1] = "";
     char fn[13+1];
     const char *f;
 
+    // Large ratios can happen because of overlays that are
+    // appended after a program is packed.
     unsigned ratio = get_ratio(fu_len, fc_len);
-    upx_snprintf(r, sizeof(r), "%3d.%02d%%", ratio / 10000, (ratio % 10000) / 100);
+#if 1
+    if (ratio >= 1000*1000)
+        strcpy(r, "overlay");
+#else
+    if (ratio >= 10*1000*1000)      // >= "1000%"
+        strcpy(r, "999.99%");
+#endif
+    else
+        upx_snprintf(r, sizeof(r), "%3d.%02d%%", ratio / 10000, (ratio % 10000) / 100);
     if (decompress)
         f = "%10ld <-%10ld  %7s  %13s  %s";
     else
@@ -172,7 +182,7 @@ UiPacker::UiPacker(const Packer *p_) :
 
     clear_cb();
 
-    s = new State;
+    s = new UiPacker__State;
     memset(s,0,sizeof(*s));
     s->msg_buf[0] = '\r';
 
@@ -271,13 +281,15 @@ void UiPacker::startCallback(unsigned u_len, unsigned step,
     // set pass
     if (total_passes > 1)
     {
+        int buflen, l;
         do {
             s->pass_digits++;
             total_passes /= 10;
         } while (total_passes > 0);
-        int l = sprintf(&s->msg_buf[s->bar_pos], "%*d/%*d  ",
-                        s->pass_digits, s->pass,
-                        s->pass_digits, s->total_passes);
+        buflen = sizeof(s->msg_buf) - s->bar_pos;
+        l = upx_snprintf(&s->msg_buf[s->bar_pos], buflen, "%*d/%*d  ",
+                         s->pass_digits, s->pass,
+                         s->pass_digits, s->total_passes);
         if (l > 0 && s->bar_len - l > 10)
         {
             s->bar_len -= l;
@@ -381,11 +393,11 @@ void UiPacker::endCallback()
 
 void __UPX_ENTRY UiPacker::callback(upx_uint isize, upx_uint osize, int state, void *user)
 {
-    //printf("%6d %6d %d\n", is, os, state);
+    //printf("%6d %6d %d\n", isize, osize, state);
     if (state != -1 && state != 3) return;
     if (user)
     {
-        UiPacker *uip = reinterpret_cast<UiPacker *>(user);
+        UiPacker *uip = (UiPacker *) user;
         uip->doCallback(isize, osize);
     }
 }
@@ -440,9 +452,10 @@ void UiPacker::doCallback(unsigned isize, unsigned osize)
     if (osize > 0)
         ratio = get_ratio(isize, osize);
 
-    sprintf(m,"  %3d.%1d%%  %c ",
-            ratio / 10000, (ratio % 10000) / 1000,
-            spinner[s->spin_counter & 3]);
+    int buflen = &s->msg_buf[sizeof(s->msg_buf)] - m;
+    upx_snprintf(m, buflen, "  %3d.%1d%%  %c ",
+                 ratio / 10000, (ratio % 10000) / 1000,
+                 spinner[s->spin_counter & 3]);
     assert((int)strlen(s->msg_buf) < 1 + 80);
 
     s->pos = pos;
@@ -536,6 +549,9 @@ void UiPacker::uiUnpackEnd(const OutputFile *fo)
 {
     uiUpdate(-1, fo->getBytesWritten());
 
+    if (s->mode == M_QUIET)
+        return;
+
     const char *name = p->fi->getName();
     if (opt->output_name)
         name = opt->output_name;
@@ -590,8 +606,9 @@ void UiPacker::uiListTotal(bool decompress)
     if (opt->verbose >= 1 && total_files >= 2)
     {
         char name[32];
-        upx_snprintf(name,sizeof(name),"[ %ld file%s ]", total_files_done, total_files_done == 1 ? "" : "s");
-        con_fprintf(stdout,"%s%s\n",
+        upx_snprintf(name, sizeof(name), "[ %ld file%s ]",
+                     total_files_done, total_files_done == 1 ? "" : "s");
+        con_fprintf(stdout, "%s%s\n",
                     header_line2,
                     mkline(total_fu_len, total_fc_len,
                            total_u_len, total_c_len,
